#include "Adc.h"


AdcClass Adc;

#define ADC_PRESCALER_MASK 7 // 125 kHz bei einem 16 MHz Arduino Nano

AdcClass::AdcClass(bool switchOn) {
   if (switchOn)
      ADCSRA = (1 << ADEN) | ADC_PRESCALER_MASK;
   else
      ADCSRA = 0;
}

void AdcClass::setReference(AdcReference refSource, uint16_t refVoltage, uint8_t delayms ) {
   this->analogReference = refSource;
   this->refVoltage = refVoltage;
   delay(delayms); // Der ADC muss sich stabilisieren
}

void AdcClass::switchOff() {
   // ADC deaktivieren
   ADCSRA = 0;
   ADMUX = 0;
}

uint16_t AdcClass::read(const AdcChannel channel) const {
   uint16_t result = 0;

   if (!(ADCSRA = (1 << ADEN))) { // falls ausgeschaltet
      ADCSRA = (1 << ADEN) | ADC_PRESCALER_MASK; // ADC einschalten
      delay(10);
   }
   ADCSRA |= ADC_PRESCALER_MASK;

   // Kanal des Multiplexers whlen
   ADMUX = ((uint8_t)analogReference << REFS0) | (uint8_t)channel;

   delay(10);

   // Den ADC initialisieren und eine Blind-Messung machen
   ADCSRA |= (1 << ADSC);
   while (ADCSRA & (1 << ADSC));

   // Messung
   ADCSRA |= (1 << ADSC);
   // Auf Ergebnis warten...
   while (ADCSRA & (1 << ADSC));

   result = ADCW;

   // Messwert zurckliefern
   return result;
}

Voltage AdcClass::readmv(const AdcChannel channel) const {
   // vcc entspricht einem ADC-Wert von 1023;
   uint32_t val = read(channel);
   return val * refVoltage / 1023;
}


uint16_t AdcClass::readAvg(const AdcChannel channel, uint8_t count) const {
   uint8_t  i;
   uint16_t result = 0;
   if (count > 63) count = 63;
   if (count == 0) count = 1;

   if (!(ADCSRA = (1 << ADEN))) { // falls ausgeschaltet
      ADCSRA = (1 << ADEN) | ADC_PRESCALER_MASK; // ADC einschalten
      delay(10);
   }
   ADCSRA |= ADC_PRESCALER_MASK;

   // Kanal des Multiplexers whlen
   ADMUX = ((uint8_t)analogReference << REFS0) | (uint8_t)channel;

   // Den ADC initialisieren und eine Blind-Messung machen
   ADCSRA |= (1 << ADSC);
   while (ADCSRA & (1 << ADSC));

   // Jetzt 'AdcAvgCount' mal die analoge Spannung and Kanal 'channel' auslesen
   // und dann Durchschnittswert ausrechnen.
   for (i = 0; i < count; i++)
   { // Eine Wandlung
      ADCSRA |= (1 << ADSC);
      // Auf Ergebnis warten...
      while (ADCSRA & (1 << ADSC));

      result += ADCW;
   }
   // Mittelwert zurckliefern
   return result / count;
}


Voltage AdcClass::readAvgmv(const AdcChannel channel, uint8_t count) const {
   // vcc entspricht einem ADC-Wert von 1023;
   uint32_t avg = readAvg(channel, count);
   return avg * refVoltage / 1023;
}

uint16_t AdcClass::readVcc() const {
   uint8_t i;
   uint16_t result = 0;
   ADMUX = 0x0E       // Bandgap als Input whlen
      | (1 << REFS0); // Referenz: AVCC

   if (!(ADCSRA = (1 << ADEN))) { // falls ausgeschaltet
      ADCSRA = (1 << ADEN) | ADC_PRESCALER_MASK; // ADC einschalten
      delay(10);
   }
   ADCSRA |= ADC_PRESCALER_MASK;


   // Dummy-Readout zur Stabilisierung der Bandgap-Spannungsquelle
   for (i = 0; i < VccDummyReads; i++)
   {
      ADCSRA |= (1 << ADSC);        // eine ADC-Wandlung
      while (ADCSRA & (1 << ADSC)); // auf Abschluss der Konvertierung warten
   }

   // Eigentliche Messung - Mittelwert aus VccReadCount aufeinanderfolgenden Wandlungen
   // Der Messwert liegt zwischen etwa 220 bei 5V und 380 bei 3 V
   for (i = 0; i < VccReadCount; i++)
   {
      ADCSRA |= (1 << ADSC);        // eine Wandlung "single conversion"
      while (ADCSRA & (1 << ADSC)); // auf Abschluss der Konvertierung warten
      result += ADCW;               // Ergebnisse aufaddieren
   }

   uint32_t r = (uint32_t)VccVbg * 1023 * VccReadCount / result;

   return r;
}
